# -*-python-*-

__package__    = "pocketMMap/toc.py"
__version__    = "0.2b"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/pocketMMap"
__date__       = "$Date: 2008/09/09 08:02:29 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import cairo
import Image

import math

class toc :

    def __init__ (self, width, height, margin=30) :
        self.page_height = height
        self.page_width = width
        self.margin = margin
        
        self.pages = []

    #
    #
    #
    
    def create (self, data) :

        (surface, ctx, x, y) = self.start_page()
    
        # why one-liners are kind of dumb; ensure
        # a constant line height...
        
        ln_height = int(max(map(lambda m: m[1], map(lambda l: self.measure_text(ctx, l[0]['label']), data))) * 1.5)
        
        data.sort(lambda m, n: cmp(m[0]['label'].lower(), n[0]['label'].lower()))
        sz_dot = self.measure_text(ctx, ".")

        #

        tags_idx = {'unfiled':[]}
        
        for stuff in data:
            tags = stuff[0]['tags']

            if len(tags) == 0 :
                tags_idx['unfiled'].append(stuff)
                
            else :
                for t in tags :

                    if not tags_idx.has_key(t):
                        tags_idx[t] = []
                    
                    tags_idx[t].append(stuff)

        tags_keys = tags_idx.keys()
        tags_keys.sort()

        tags_data = []
        
        for t in tags_keys :
            tags_data.append([{'header':t}])

            for place in tags_idx[t] :
                tags_data.append(place)

        #
        #
        #
        
        for stuff in tags_data:

            if stuff[0].has_key('header') :

                header = stuff[0]['header'].upper()
                sz_header = self.measure_text(ctx, header)

                y += int(ln_height / 2)
                
                ctx.move_to(x, y)
                ctx.show_text(header)

                y += int(ln_height / 2)
                
            else :
                label = stuff[0]['label']
                tags = stuff[0]['tags']
                
                page = stuff[1]
                marker = stuff[2]

                sz_label = self.measure_text(ctx, label)
                
                loc = " %s / %s" % (page, marker)
                sz_loc = self.measure_text(ctx, loc)
                
                stop = self.page_width - (sz_loc[0] + self.margin)    
                
                while sz_label[0] > (stop - sz_loc[0]) :
                    label = label[:-1]
                    sz_label = self.measure_text(ctx, label)

                ctx.move_to(x, y)
                ctx.show_text(unicode(label))

                x = x + (sz_label[0] + (sz_dot[0] * 2))

                while x < stop :
                    ctx.move_to(x, y)
                    ctx.show_text(".")
                    x = x + (sz_dot[0] * 2)
                    
                ctx.move_to(x, y)
                ctx.show_text(loc)

            x = self.margin
            y = y + ln_height

            if y > (self.page_height - (self.margin * 2)) :
                self.write_page(surface)
                (surface, ctx, x, y) = self.start_page()    

        self.write_page(surface)
        return self.pages
        
    #
    #
    #
    
    def measure_text (self, ctx, txt):
        
        xbearing, ybearing, width, height, xadvance, yadvance = (
            ctx.text_extents(unicode(txt)))
        
        return (width, height)

    #
    #
    #

    def start_page (self) :

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.page_width, self.page_height)
        ctx = cairo.Context(surface)

        # the pycairo docs are kind of useless sometimes
        # http://www.gnome.org/~federico/docs/file-chooser-profile/code/plot-languages.py
        
        font_face = "Helvetica Bold"
        font_size = 11

        ctx.select_font_face(font_face, cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        ctx.set_font_size(font_size)

        ctx.move_to(0, 0)
        ctx.rectangle(0, 0, self.page_width, self.page_height)
        ctx.set_source_rgb(255, 255, 255)
        ctx.fill()

        ctx.set_source_rgb(0, 0, 0)

        x = self.margin
        y = self.margin * 2

        legend = "page / marker"
        lg_bar = self.measure_text(ctx, legend)

        x = self.page_width - int(lg_bar[0] + self.margin)
        
        ctx.move_to(x, y)
        ctx.show_text(legend)

        x = self.margin
        y = y + self.margin

        ctx.move_to(x, y)

        return (surface, ctx, x, y)

    #
    #
    #
    
    def write_page (self, surface) :
        png = self.cairo2pil(surface)
        self.pages.append(png)

    #
    #
    #
    
    def cairo2pil(self, surface) :

        mode='RGBA'

        width = surface.get_width()
        height = surface.get_height()

        return Image.frombuffer(mode, (width, height), surface.get_data(), "raw", mode, 0, 1)

    #
    #
    #
    
if __name__ == '__main__' :

    data = [
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 127, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        
        ]

    t = toc(360, 200)
    
